/***********************************************************************/
/*                                                                     */
/*  FILE        : RIIC_RX210_SLAVE.c                                   */
/*  DATE        : Tue, Oct 31, 2006                                    */
/*  DESCRIPTION : Main Program                                         */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "typedefine.h"
#include "iodefine.h"
#include <stdio.h>
#include <machine.h>

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define SLAVE_RECEIVES_DATA                  (0x00U) /* Slave receiving data */
#define SLAVE_SENDS_DATA                     (0x01U) /* Slave sending data */
#define SLAVE_SENDS_END                      (0x02U) /* Slave send prepares to stop */
#define SLAVE_SENDS_STOP                     (0x03U) /* Slave send stopping */
#define SLAVE_RECEIVES_STOP                  (0x04U) /* Slave receive stopping */
#define SLAVE_WAIT_START_CONDITION           (0x05U) /* Slave is waiting for start condition */
#define SLAVE_RECEIVE                        (0x06U) /* Slave receive mode */
#define SLAVE_TRANSMIT                       (0x07U) /* Slave transmit mode */
#define SLAVE_RECEIVE_END                    (0x08U) /* Slave receive mode */
#define BUFFER_SIZE                          (0x20U)
#define RECEIVE_SIZE                         (5U)

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
unsigned short riic0_mode_flag;
unsigned short riic0_state;
unsigned char* riic0_rx_address;
unsigned char* riic0_tx_address;
unsigned short riic0_rx_count;
unsigned short riic0_tx_count;
unsigned short riic0_rx_length;
unsigned short riic0_dummy_read_count;
unsigned char  riic0_buffer[BUFFER_SIZE];


void main(void);
void cgc_init(void);
void port_init(void);
void riic0_slave_init(void);
void riic0_slave_send(unsigned char * const tx_buf, unsigned short tx_num);
void riic0_slave_receive(unsigned char * const rx_buf, unsigned short  rx_num);

/*******************************************************************************
* Function Name: main
* Description  : Main program
* Arguments    : none
* Return Value : none
*******************************************************************************/
void main(void)
{
    cgc_init();
    port_init();
    riic0_slave_init();

    /* RIIC0 Start */
    IR(RIIC0,TXI0) = 0U;
    IR(RIIC0,RXI0) = 0U;
    IEN(RIIC0,TXI0) = 1U;
    IEN(RIIC0,RXI0) = 1U;
    IEN(RIIC0,EEI0) = 1U;

    /* Receive Wait */
    riic0_slave_receive(riic0_buffer, RECEIVE_SIZE);

    /* Enable Signal to RX63N */
    PORT2.PODR.BIT.B1 = 0U;
    PORT2.PODR.BIT.B1 = 1U;

    while (1U)
    {
        /* transmit end */
        if (SLAVE_RECEIVE_END == riic0_mode_flag)
        {
            /* Send receive data to master device */
            riic0_slave_send(riic0_buffer,riic0_rx_count);
            PORT2.PODR.BIT.B1 = 0U;
            break;
        }
        else
        {
            /* Do nothing */
        }
    }

    while(1);
}

/*******************************************************************************
* Function Name: cgc_init
* Description  : Main program
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cgc_init(void)
{
    unsigned long ii = 0U;

    /* Enable writing to registers related to operating modes, LPC, CGC and software reset */
    SYSTEM.PRCR.WORD = 0xA50B;

    /* Set main clock control registers */
    SYSTEM.MOFCR.BYTE = 0x20;
    SYSTEM.MOSCWTCR.BYTE = 0x0A;

    /* Set main clock operation */
    SYSTEM.MOSCCR.BIT.MOSTP = 0U;

    while (0x00 != SYSTEM.MOSCCR.BYTE) 
    {
        /* Confirm that the written value can be read correctly. */
    }

    for(ii = 0; ii < 0x27D; ii++)
    {
        /* Wait 6ms. See comment above for why. */
        nop() ;
    }

    /* Set the sub-clock to stopped. */
    SYSTEM.SOSCCR.BYTE = 0x01;

    /*
    Clock Description              Frequency
    ----------------------------------------
    Input Clock Frequency............. 20 MHz
    Peripheral Clock B Frequency...... 20 MHz
    System Clock Frequency............ 20 MHz */
    SYSTEM.SCKCR.LONG = 0x00000000;

    while (0x00000000 != SYSTEM.SCKCR.LONG) 
    {
         /* Confirm that the written value can be read correctly. */
    }

    /* Choose clock source (Main) */
    SYSTEM.SCKCR3.WORD = 0x0200;

    while (0x0200 != SYSTEM.SCKCR3.WORD) 
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* LOCO is not chosen. This is the default out of reset. */
    SYSTEM.LOCOCR.BIT.LCSTP = 1U;

    /* Enable write protection */
    SYSTEM.PRCR.WORD = 0xA500;
}

/*******************************************************************************
* Function Name: port_init
* Description  : Main program
* Arguments    : none
* Return Value : none
*******************************************************************************/
void port_init(void)
{
    PORT2.PODR.BIT.B1 = 1U;  /* P21 High */
    PORT2.PDR.BIT.B1  = 1U;  /* P21 Output */
}

/*******************************************************************************
* Function Name: riic0_slave_init
* Description  : Initialize RIIC0
* Arguments    : none
* Return Value : none
*******************************************************************************/
void riic0_slave_init(void)
{
    /* Enable writing to registers related to operating modes, LPC, CGC and software reset */
    SYSTEM.PRCR.WORD = 0xA502;

    /* Enable writing to MPC pin function control registers */
    MPC.PWPR.BIT.B0WI  = 0U;
    MPC.PWPR.BIT.PFSWE = 1U;

    /* ----- RIIC0 Initialize Start ----- */
    MSTP(RIIC0) = 0U;

    RIIC0.ICCR1.BIT.ICE    = 0U;
    RIIC0.ICCR1.BIT.IICRST = 1U;
    RIIC0.ICCR1.BIT.ICE    = 1U;

    /* Set SARLy and SARUy */
    RIIC0.SARU0.BYTE = 0x00;
    RIIC0.SARL0.BYTE = 0x34U;

    /* Set ICSER */
    RIIC0.ICSER.BYTE = 0x01;

    /* Set transfer bit rate(100kbps) */
    RIIC0.ICMR1.BYTE = 0x20;
    RIIC0.ICBRL.BYTE = 0xFF;
    RIIC0.ICBRH.BYTE = 0xEB;

    /* Set ICMR2 and ICMR3 */
    RIIC0.ICMR2.BYTE = 0x00;

    /* Set ICFER */
    RIIC0.ICFER.BYTE = 0x0C;

    /* Set ICIER */
    RIIC0.ICIER.BYTE = 0xEE;

    /* Cancel internal reset */
    RIIC0.ICCR1.BIT.IICRST = 0U;

    /* Set interrupt priority */
    IPR(RIIC0, TXI0) = 0x0F;
    IPR(RIIC0, TEI0) = 0x0F;
    IPR(RIIC0, RXI0) = 0x0F;
    IPR(RIIC0, EEI0) = 0x0F;

    /* Set SCL0 pin */
    MPC.P12PFS.BYTE = 0x0FU;
    PORT1.PMR.BYTE |= 0x04U;

    /* Set SDA0 pin */
    MPC.P13PFS.BYTE = 0x0FU;
    PORT1.PMR.BYTE |= 0x08U;
    /* ----- RIIC0 Initialize End ----- */

    /* Disable writing to MPC pin function control registers */
    MPC.PWPR.BIT.PFSWE = 0U;
    MPC.PWPR.BIT.B0WI = 1U;

    /* Enable protection */
    SYSTEM.PRCR.WORD = 0xA500;
}

/*******************************************************************************
* Function Name: riic0_slave_send
* Description  : Setting slave transmitting
* Arguments    : txbuf -
*                    transmit buffer pointer
*                txnum -
*                    transmit data length
* Return Value : none
*******************************************************************************/
void riic0_slave_send(unsigned char * const tx_buf, unsigned short tx_num)
{
    riic0_tx_count        = tx_num;
    riic0_tx_address      = tx_buf;
    RIIC0.ICMR3.BYTE     |= 0x60U;
    RIIC0.ICMR3.BIT.ACKWP = 1U;
    RIIC0.ICMR3.BIT.ACKBT = 0U;

    riic0_mode_flag = SLAVE_TRANSMIT;
    riic0_state = SLAVE_WAIT_START_CONDITION;
}

/*******************************************************************************
* Function Name: riic0_slave_receive
* Description  : Setting slave receiving
* Arguments    : rxbuf -
*                    receive buffer pointer
*                rxnum -
*                    receive data length
* Return Value : none
*******************************************************************************/
void riic0_slave_receive(unsigned char * const rx_buf, unsigned short rx_num)
{
    riic0_rx_length = rx_num;
    riic0_rx_count = 0U;
    riic0_rx_address = rx_buf;
    RIIC0.ICMR3.BYTE |= 0x60U;
    RIIC0.ICMR3.BIT.ACKWP = 1U;
    RIIC0.ICMR3.BIT.ACKBT = 0U;
    riic0_dummy_read_count = 0U;

    riic0_mode_flag = SLAVE_RECEIVE;
    riic0_state = SLAVE_WAIT_START_CONDITION;
}

/**********************************************************************************************************************
* Function Name: r_riic0_transmit_interrupt
* Description  : This function is TXI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_transmit_interrupt(vect=VECT(RIIC0,TXI0))
static void r_riic0_transmit_interrupt(void)
{
    /* Slave transmit */
    if (SLAVE_TRANSMIT == riic0_mode_flag)
    {
        /* Slave transmit data */
        if (SLAVE_SENDS_DATA == riic0_state)
        {
            /* write transmitting data */
            RIIC0.ICDRT = *riic0_tx_address;
            riic0_tx_address++;
            riic0_tx_count--;

            /* Transmit last data */
            if (0U == riic0_tx_count)
            {
                /* Disable I2C transmit interrupt */
                IEN(RIIC0,TXI0) = 0U;
                /* Enable I2C transmit end interrupt */
                IEN(RIIC0,TEI0) = 1U;
                /* write transmitting data */
                RIIC0.ICDRT = *riic0_tx_address;
                riic0_state = SLAVE_SENDS_END;
            }
            else
            {
                /* Do nothing */
            }
        }
        else
        {
            /* Do nothing */
        }
    }
    else
    {
        /* Do nothing */
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_transmitend_interrupt
* Description  : This function is TEI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_transmitend_interrupt(vect=VECT(RIIC0,TEI0))
static void r_riic0_transmitend_interrupt(void)
{
    volatile unsigned char dummy;

    /* Slave transmit complete */
    if (SLAVE_SENDS_END == riic0_state)
    {
        /* Disable I2C transmit end interrupt */
        IEN(RIIC0,TEI0) = 0U;
        riic0_state = SLAVE_SENDS_STOP;

        /* Dummy read to release SCL */
        dummy = RIIC0.ICDRR;
    }
    else
    {
        /* Do nothing */
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_receive_interrupt
* Description  : This function is RXI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_receive_interrupt(vect=VECT(RIIC0,RXI0))
static void r_riic0_receive_interrupt(void)
{
    volatile unsigned char dummy;

    /* Slave receiving */
    if (SLAVE_RECEIVES_DATA == riic0_state)
    {
        /* Dummy read */
        if (1U > riic0_dummy_read_count)
        {
            dummy = RIIC0.ICDRR;
            riic0_dummy_read_count++;
            return;
        }
        else
        {
            /* Do nothing */
        }

        /* Setting ACK */
        RIIC0.ICMR3.BIT.ACKWP = 1U;
        RIIC0.ICMR3.BIT.ACKBT = 0U;

        /* read receiving data */
        *riic0_rx_address = RIIC0.ICDRR;
        riic0_rx_address++;
        riic0_rx_count++;

        /* Stop condition */
        if (1U == RIIC0.ICSR2.BIT.STOP)
        {
            /* check stop request */
            riic0_state = SLAVE_RECEIVES_STOP;
        }
        /* Receiving end */
        else if (riic0_rx_count == riic0_rx_length)
        {
            riic0_state = SLAVE_RECEIVES_STOP;
        }
        else
        {
            /* Do nothing */
        }
    }
    else
    {
        /* Do nothing */
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_error_interrupt
* Description  : This function is EEI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_error_interrupt(vect=VECT(RIIC0,EEI0))
static void r_riic0_error_interrupt(void)
{
    /* Slave receiving */
    if (SLAVE_RECEIVE == riic0_mode_flag)
    {
        /* Slave receive stop */
        if (SLAVE_RECEIVES_STOP == riic0_state)
        {
            RIIC0.ICSR2.BIT.STOP = 0U;
            RIIC0.ICIER.BIT.SPIE = 0U;
            RIIC0.ICIER.BIT.STIE = 1U;

            riic0_mode_flag = SLAVE_RECEIVE_END;
        }
        /* Slave wait start condition */
        else if (SLAVE_WAIT_START_CONDITION == riic0_state)
        {
            RIIC0.ICSR2.BIT.START = 0U;
            RIIC0.ICIER.BIT.STIE = 0U;
            RIIC0.ICIER.BIT.SPIE = 1U;

            riic0_state = SLAVE_RECEIVES_DATA;
        }
        else
        {
            /* Do nothing */
        }
    }
    /* Slave transmiting */
    else if (SLAVE_TRANSMIT == riic0_mode_flag)
    {
        /* Stop slave transmit */
        if (SLAVE_SENDS_STOP == riic0_state)
        {
            RIIC0.ICSR2.BIT.NACKF = 0U;
            RIIC0.ICSR2.BIT.STOP = 0U;
            RIIC0.ICIER.BIT.SPIE = 0U;
            RIIC0.ICIER.BIT.STIE = 1U;
        }
        /* Slave wait start condition */
        else if (SLAVE_WAIT_START_CONDITION == riic0_state)
        {
            RIIC0.ICSR2.BIT.START = 0U;
            RIIC0.ICIER.BIT.STIE = 0U;
            RIIC0.ICIER.BIT.SPIE = 1U;

            riic0_state = SLAVE_SENDS_DATA;
        }
        else
        {
            /* Do nothing */
        }
    }
    else
    {
        /* Do nothing */
    }
}
